@@ -19,8 +19,9 @@
 /*	Append a recipient (non-)delivery status record to a per-message
 /*	log file.
 /* .IP \(bu
-/*	Enqueue a bounce message, with a copy of a per-message log file
-/*	and of the corresponding message. When the bounce message is
+/*	Enqueue a delivery status notification message, with a copy
+/*	of a per-message log file and of the corresponding message.
+/*	When the delivery status notification message is
 /*	enqueued successfully, the per-message log file is deleted.
 /* .PP
 /*	The software does a best notification effort. A non-delivery
@@ -33,10 +34,11 @@
 /*	themselves, and that depend on retry logic in their own client.
 /* STANDARDS
 /*	RFC 822 (ARPA Internet Text Messages)
+/*	RFC 2045 (Format of Internet Message Bodies)
 /*	RFC 2822 (ARPA Internet Text Messages)
 /*	RFC 3462 (Delivery Status Notifications)
 /*	RFC 3464 (Delivery Status Notifications)
-/*	RFC 2045 (Format of Internet Message Bodies)
+/*	RFC 3834 (Auto-Submitted: message header)
 /* DIAGNOSTICS
 /*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /* CONFIGURATION PARAMETERS
@@ -82,15 +84,19 @@
 /* .IP "\fBipc_timeout (3600s)\fR"
 /*	The time limit for sending or receiving information over an internal
 /*	communication channel.
+/* .IP "\fBinternal_mail_filter_classes (empty)\fR"
+/*	What categories of Postfix-generated mail are subject to
+/*	before-queue content inspection by non_smtpd_milters, header_checks
+/*	and body_checks.
 /* .IP "\fBmail_name (Postfix)\fR"
 /*	The mail system name that is displayed in Received: headers, in
 /*	the SMTP greeting banner, and in bounced mail.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of connection requests before a Postfix daemon
-/*	process terminates.
+/*	The maximal number of incoming connections that a Postfix daemon
+/*	process will service before terminating voluntarily.
 /* .IP "\fBnotify_classes (resource, software)\fR"
 /*	The list of error classes that are reported to the postmaster.
 /* .IP "\fBprocess_id (read-only)\fR"
@@ -149,6 +155,7 @@
 #include <mail_proto.h>
 #include <mail_queue.h>
 #include <mail_params.h>
+#include <mail_version.h>
 #include <mail_conf.h>
 #include <bounce.h>
 #include <mail_addr.h>
@@ -200,7 +207,7 @@ BOUNCE_TEMPLATES *bounce_templates;
 
 static int bounce_append_proto(char *service_name, VSTREAM *client)
 {
-    char   *myname = "bounce_append_proto";
+    const char *myname = "bounce_append_proto";
     int     flags;
 
     /*
@@ -270,7 +277,7 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
 					        char *, char *, char *, int,
 						        BOUNCE_TEMPLATES *))
 {
-    char   *myname = "bounce_notify_proto";
+    const char *myname = "bounce_notify_proto";
     int     flags;
     int     dsn_ret;
 
@@ -327,7 +334,7 @@ static int bounce_notify_proto(char *service_name, VSTREAM *client,
 
 static int bounce_verp_proto(char *service_name, VSTREAM *client)
 {
-    char   *myname = "bounce_verp_proto";
+    const char *myname = "bounce_verp_proto";
     int     flags;
     int     dsn_ret;
 
@@ -399,7 +406,7 @@ static int bounce_verp_proto(char *service_name, VSTREAM *client)
 
 static int bounce_one_proto(char *service_name, VSTREAM *client)
 {
-    char   *myname = "bounce_one_proto";
+    const char *myname = "bounce_one_proto";
     int     flags;
     int     dsn_ret;
 
@@ -571,8 +578,8 @@ static void post_jail_init(char *service_name, char **unused_argv)
     /*
      * Special case: dump bounce templates. This is not part of the master(5)
      * public interface. This internal interface is used by the postconf
-     * command. It was implemented before bounce templates were isolated
-     * into modules that could have been called directly.
+     * command. It was implemented before bounce templates were isolated into
+     * modules that could have been called directly.
      */
     if (strcmp(service_name, "dump_templates") == 0) {
 	bounce_templates_dump(VSTREAM_OUT, bounce_templates);
@@ -599,6 +606,8 @@ static void post_jail_init(char *service_name, char **unused_argv)
     dsn_buf = dsb_create();
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - the main program */
 
 int     main(int argc, char **argv)
@@ -621,6 +630,11 @@ int     main(int argc, char **argv)
 	0,
     };
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     /*
      * Pass control to the single-threaded service skeleton.
      */
