@@ -67,7 +67,7 @@ typedef struct MILTERS {
     const char *(*ins_header) (void *, ssize_t, char *, char *);
     const char *(*add_rcpt) (void *, char *);
     const char *(*del_rcpt) (void *, char *);
-    const char *(*repl_body) (void *, VSTRING *);
+    const char *(*repl_body) (void *, int, VSTRING *);
 } MILTERS;
 
 typedef const char *(*MILTER_MAC_LOOKUP_FN) (const char *, void *);
@@ -75,7 +75,7 @@ typedef const char *(*MILTER_ADD_HEADER_FN) (void *, char *, char *);
 typedef const char *(*MILTER_EDIT_HEADER_FN) (void *, ssize_t, char *, char *);
 typedef const char *(*MILTER_DEL_HEADER_FN) (void *, ssize_t, char *);
 typedef const char *(*MILTER_EDIT_RCPT_FN) (void *, char *);
-typedef const char *(*MILTER_EDIT_BODY_FN) (void *, VSTRING *);
+typedef const char *(*MILTER_EDIT_BODY_FN) (void *, int, VSTRING *);
 
 extern MILTERS *milter_create(const char *, int, int, int,
 			              const char *, const char *,
@@ -99,11 +99,17 @@ extern const char *milter_unknown_event(MILTERS *, const char *);
 extern const char *milter_other_event(MILTERS *);
 extern void milter_abort(MILTERS *);
 extern void milter_disc_event(MILTERS *);
-extern int milter_dummy(MILTERS *, VSTREAM *);
 extern int milter_send(MILTERS *, VSTREAM *);
 extern MILTERS *milter_receive(VSTREAM *, int);
 extern void milter_free(MILTERS *);
 
+ /*
+  * Milter body edit commands.
+  */
+#define MILTER_BODY_START	1	/* start message body */
+#define MILTER_BODY_LINE	2	/* message body line */
+#define MILTER_BODY_END		3	/* end message body */
+
  /*
   * Sendmail 8 macro names. We support forms with and without the {}.
   */
