@@ -29,9 +29,10 @@
 /*
 /*	To prevent Postfix from sending multiple recipients per delivery
 /*	request, specify
-/*
-/* .ti +4
-/*	\fItransport\fB_destination_recipient_limit = 1\fR
+/* .sp
+/* .nf
+/*	    \fItransport\fB_destination_recipient_limit = 1\fR
+/* .fi
 /*
 /*	in the Postfix \fBmain.cf\fR file, where \fItransport\fR
 /*	is the name in the first column of the Postfix \fBmaster.cf\fR
@@ -118,8 +119,8 @@
 /*	by, for example, \fBUUCP\fR software.
 /* .RE
 /* .IP "\fBnull_sender\fR=\fIreplacement\fR (default: MAILER-DAEMON)"
-/*	Replace the null sender address, which is typically used
-/*	for delivery status notifications, with the specified text
+/*	Replace the null sender address (typically used for delivery
+/*	status notifications) with the specified text
 /*	when expanding the \fB$sender\fR command-line macro, and
 /*	when generating a From_ or Return-Path: message header.
 /*
@@ -135,17 +136,19 @@
 /*	Caution: a null sender address is easily mis-parsed by
 /*	naive software. For example, when the \fBpipe\fR(8) daemon
 /*	executes a command such as:
-/*
-/* .ti +4
-/*	command -f$sender -- $recipient (\fIbad\fR)
-/*
+/* .sp
+/* .nf
+/*	    command -f$sender -- $recipient (\fIbad\fR)
+/* .fi
+/* .IP
 /*	the command will mis-parse the -f option value when the
 /*	sender address is a null string.  For correct parsing,
 /*	specify \fB$sender\fR as an argument by itself:
-/*
-/* .ti +4
-/*	command -f $sender -- $recipient (\fIgood\fR)
-/*
+/* .sp
+/* .nf
+/*	    command -f $sender -- $recipient (\fIgood\fR)
+/* .fi
+/* .IP
 /*	This feature is available with Postfix 2.3 and later.
 /* .IP "\fBsize\fR=\fIsize_limit\fR (optional)"
 /*	Messages greater in size than this limit (in bytes) will
@@ -303,6 +306,10 @@
 /*	Limit the time for delivery to external command, for delivery via
 /*	the named \fItransport\fR.
 /*	The limit is enforced by the pipe delivery agent.
+/*
+/*	Postfix 2.4 and later support a suffix that specifies the
+/*	time unit: s (seconds), m (minutes), h (hours), d (days),
+/*	w (weeks). The default time unit is seconds.
 /* MISCELLANEOUS CONTROLS
 /* .ad
 /* .fi
@@ -325,11 +332,11 @@
 /*	The UNIX system account that owns the Postfix queue and most Postfix
 /*	daemon processes.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process
-/*	waits for the next service request before exiting.
+/*	The maximum amount of time that an idle Postfix daemon process waits
+/*	for an incoming connection before terminating voluntarily.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of connection requests before a Postfix daemon
-/*	process terminates.
+/*	The maximal number of incoming connections that a Postfix daemon
+/*	process will service before terminating voluntarily.
 /* .IP "\fBprocess_id (read-only)\fR"
 /*	The process ID of a Postfix command or daemon process.
 /* .IP "\fBprocess_name (read-only)\fR"
@@ -396,6 +403,7 @@
 #include <recipient_list.h>
 #include <deliver_request.h>
 #include <mail_params.h>
+#include <mail_version.h>
 #include <mail_conf.h>
 #include <bounce.h>
 #include <defer.h>
@@ -708,13 +716,13 @@ static ARGV *expand_argv(const char *service, char **argv,
 
 static void get_service_params(PIPE_PARAMS *config, char *service)
 {
-    char   *myname = "get_service_params";
+    const char *myname = "get_service_params";
 
     /*
      * Figure out the command time limit for this transport.
      */
     config->time_limit =
-	get_mail_conf_int2(service, "_time_limit", var_command_maxtime, 1, 0);
+	get_mail_conf_time2(service, "_time_limit", var_command_maxtime, 's', 1, 0);
 
     /*
      * Give the poor tester a clue of what is going on.
@@ -727,7 +735,7 @@ static void get_service_params(PIPE_PARAMS *config, char *service)
 
 static void get_service_attr(PIPE_ATTR *attr, char **argv)
 {
-    char   *myname = "get_service_attr";
+    const char *myname = "get_service_attr";
     struct passwd *pwd;
     struct group *grp;
     char   *user;			/* user name */
@@ -966,7 +974,7 @@ static int eval_command_status(int command_status, char *service,
 
 static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
 {
-    char   *myname = "deliver_message";
+    const char *myname = "deliver_message";
     static PIPE_PARAMS conf;
     static PIPE_ATTR attr;
     RECIPIENT_LIST *rcpt_list = &request->rcpt_list;
@@ -1192,6 +1200,8 @@ static void pre_init(char *unused_name, char **unused_argv)
     flush_init();
 }
 
+MAIL_VERSION_STAMP_DECLARE;
+
 /* main - pass control to the single-threaded skeleton */
 
 int     main(int argc, char **argv)
@@ -1201,6 +1211,11 @@ int     main(int argc, char **argv)
 	0,
     };
 
+    /*
+     * Fingerprint executables and core dumps.
+     */
+    MAIL_VERSION_STAMP_ALLOCATE;
+
     single_server_main(argc, argv, pipe_service,
 		       MAIL_SERVER_TIME_TABLE, time_table,
 		       MAIL_SERVER_PRE_INIT, pre_init,
