@@ -56,12 +56,12 @@ typedef struct QMGR_SCAN QMGR_SCAN;
 }
 
 #define QMGR_LIST_UNLINK(head, type, object, peers) { \
-    type   next = object->peers.next; \
-    type   prev = object->peers.prev; \
-    if (prev) prev->peers.next = next; \
-    else head.next = next; \
-    if (next) next->peers.prev = prev; \
-    else head.prev = prev; \
+    type   _next = object->peers.next; \
+    type   _prev = object->peers.prev; \
+    if (_prev) _prev->peers.next = _next; \
+	else head.next = _next; \
+    if (_next) _next->peers.prev = _prev; \
+	else head.prev = _prev; \
     object->peers.next = object->peers.prev = 0; \
 }
 
@@ -131,6 +131,7 @@ struct QMGR_JOB_LIST {
 
 struct QMGR_TRANSPORT {
     int     flags;			/* blocked, etc. */
+    int     pending;			/* incomplete DA connections */
     char   *name;			/* transport name */
     int     dest_concurrency_limit;	/* concurrency per domain */
     int     init_dest_concurrency;	/* init. per-domain concurrency */
@@ -138,6 +139,9 @@ struct QMGR_TRANSPORT {
     int     rcpt_per_stack;		/* extra slots reserved for jobs put
 					 * on the job stack */
     int     rcpt_unused;		/* available in-core recipient slots */
+    int     refill_limit;		/* recipient batch size for message
+					 * refill */
+    int     refill_delay;		/* delay before message refill */
     int     slot_cost;			/* cost of new preemption slot (# of
 					 * selected entries) */
     int     slot_loan;			/* preemption boost offset and */
@@ -163,7 +167,6 @@ struct QMGR_TRANSPORT {
 };
 
 #define QMGR_TRANSPORT_STAT_DEAD	(1<<1)
-#define QMGR_TRANSPORT_STAT_BUSY	(1<<2)
 
 typedef void (*QMGR_TRANSPORT_ALLOC_NOTIFY) (QMGR_TRANSPORT *, VSTREAM *);
 extern QMGR_TRANSPORT *qmgr_transport_select(void);
@@ -173,6 +176,8 @@ extern void qmgr_transport_unthrottle(QMGR_TRANSPORT *);
 extern QMGR_TRANSPORT *qmgr_transport_create(const char *);
 extern QMGR_TRANSPORT *qmgr_transport_find(const char *);
 
+#define QMGR_TRANSPORT_THROTTLED(t)	((t)->flags & QMGR_TRANSPORT_STAT_DEAD)
+
  /*
   * Each next hop (e.g., a domain name) has its own queue of pending message
   * transactions. The "todo" queue contains messages that are to be delivered
@@ -213,6 +218,8 @@ extern void qmgr_queue_throttle(QMGR_QUEUE *, DSN *);
 extern void qmgr_queue_unthrottle(QMGR_QUEUE *);
 extern QMGR_QUEUE *qmgr_queue_find(QMGR_TRANSPORT *, const char *);
 
+#define QMGR_QUEUE_THROTTLED(q) ((q)->window <= 0)
+
  /*
   * Structure of one next-hop queue entry. In order to save some copying
   * effort we allow multiple recipients per transaction.
@@ -229,6 +236,7 @@ struct QMGR_ENTRY {
 
 extern QMGR_ENTRY *qmgr_entry_select(QMGR_PEER *);
 extern void qmgr_entry_unselect(QMGR_ENTRY *);
+extern void qmgr_entry_move_todo(QMGR_QUEUE *, QMGR_ENTRY *);
 extern void qmgr_entry_done(QMGR_ENTRY *, int);
 extern QMGR_ENTRY *qmgr_entry_create(QMGR_PEER *, QMGR_MESSAGE *);
 
@@ -252,6 +260,8 @@ struct QMGR_MESSAGE {
     struct timeval active_time;		/* time of entry into active queue */
     time_t  queued_time;		/* sanitized time when moved to the
 					 * active queue */
+    time_t  refill_time;		/* sanitized time of last message
+					 * refill */
     long    warn_offset;		/* warning bounce flag offset */
     time_t  warn_time;			/* time next warning to be sent */
     long    data_offset;		/* data seek offset */
@@ -265,7 +275,8 @@ struct QMGR_MESSAGE {
     char   *filter_xport;		/* filtering transport */
     char   *inspect_xport;		/* inspecting transport */
     char   *redirect_addr;		/* info@spammer.tld */
-    long    data_size;			/* message content size */
+    long    data_size;			/* data segment size */
+    long    cont_length;		/* message content length */
     long    rcpt_offset;		/* more recipients here */
     char   *client_name;		/* client hostname */
     char   *client_addr;		/* client address */
@@ -296,7 +307,7 @@ extern int qmgr_recipient_count;
 extern void qmgr_message_free(QMGR_MESSAGE *);
 extern void qmgr_message_update_warn(QMGR_MESSAGE *);
 extern void qmgr_message_kill_record(QMGR_MESSAGE *, long);
-extern QMGR_MESSAGE *qmgr_message_alloc(const char *, const char *, int);
+extern QMGR_MESSAGE *qmgr_message_alloc(const char *, const char *, int, mode_t);
 extern QMGR_MESSAGE *qmgr_message_realloc(QMGR_MESSAGE *);
 
 #define QMGR_MSG_STATS(stats, message) \
@@ -359,6 +370,7 @@ extern void qmgr_job_move_limits(QMGR_JOB *);
 
 extern QMGR_PEER *qmgr_peer_create(QMGR_JOB *, QMGR_QUEUE *);
 extern QMGR_PEER *qmgr_peer_find(QMGR_JOB *, QMGR_QUEUE *);
+extern QMGR_PEER *qmgr_peer_obtain(QMGR_JOB *, QMGR_QUEUE *);
 extern void qmgr_peer_free(QMGR_PEER *);
 
  /*
@@ -414,7 +426,9 @@ struct QMGR_SCAN {
   */
 #define QMGR_SCAN_START	(1<<0)		/* start now/restart when done */
 #define QMGR_SCAN_ALL	(1<<1)		/* all queue file time stamps */
-#define QMGR_FLUSH_DEAD	(1<<2)		/* all sites, all transports */
+#define QMGR_FLUSH_ONCE	(1<<2)		/* unthrottle once */
+#define QMGR_FLUSH_DFXP	(1<<3)		/* override defer_transports */
+#define QMGR_FLUSH_EACH	(1<<4)		/* unthrottle per message */
 
  /*
   * qmgr_scan.c
@@ -423,6 +437,13 @@ extern QMGR_SCAN *qmgr_scan_create(const char *);
 extern void qmgr_scan_request(QMGR_SCAN *, int);
 extern char *qmgr_scan_next(QMGR_SCAN *);
 
+ /*
+  * qmgr_error.c
+  */
+extern QMGR_TRANSPORT *qmgr_error_transport(const char *);
+extern QMGR_QUEUE *qmgr_error_queue(const char *, DSN *);
+extern char *qmgr_error_nexthop(DSN *);
+
 /* LICENSE
 /* .ad
 /* .fi
