@@ -124,7 +124,7 @@ static void qmgr_active_done_3_generic(QMGR_MESSAGE *);
 
 static void qmgr_active_corrupt(const char *queue_id)
 {
-    char   *myname = "qmgr_active_corrupt";
+    const char *myname = "qmgr_active_corrupt";
 
     if (mail_queue_rename(queue_id, MAIL_QUEUE_ACTIVE, MAIL_QUEUE_CORRUPT)) {
 	if (errno != ENOENT)
@@ -143,7 +143,7 @@ static void qmgr_active_corrupt(const char *queue_id)
 static void qmgr_active_defer(const char *queue_name, const char *queue_id,
 			              const char *dest_queue, int delay)
 {
-    char   *myname = "qmgr_active_defer";
+    const char *myname = "qmgr_active_defer";
     const char *path;
     struct utimbuf tbuf;
 
@@ -169,7 +169,7 @@ static void qmgr_active_defer(const char *queue_name, const char *queue_id,
 
 int     qmgr_active_feed(QMGR_SCAN *scan_info, const char *queue_id)
 {
-    char   *myname = "qmgr_active_feed";
+    const char *myname = "qmgr_active_feed";
     QMGR_MESSAGE *message;
     struct stat st;
     const char *path;
@@ -226,8 +226,15 @@ int     qmgr_active_feed(QMGR_SCAN *scan_info, const char *queue_id)
      * being delivered. In that case (the file is locked), defer delivery by
      * a minimal amount of time.
      */
+#define QMGR_FLUSH_AFTER	(QMGR_FLUSH_EACH | QMGR_FLUSH_DFXP)
+
     if ((message = qmgr_message_alloc(MAIL_QUEUE_ACTIVE, queue_id,
-				      scan_info->flags)) == 0) {
+				 (st.st_mode & MAIL_QUEUE_STAT_UNTHROTTLE) ?
+				      scan_info->flags | QMGR_FLUSH_AFTER :
+				      scan_info->flags,
+				 (st.st_mode & MAIL_QUEUE_STAT_UNTHROTTLE) ?
+				  st.st_mode & ~MAIL_QUEUE_STAT_UNTHROTTLE :
+				      0)) == 0) {
 	qmgr_active_corrupt(queue_id);
 	return (0);
     } else if (message == QMGR_MESSAGE_LOCKED) {
@@ -249,7 +256,7 @@ int     qmgr_active_feed(QMGR_SCAN *scan_info, const char *queue_id)
 
 void    qmgr_active_done(QMGR_MESSAGE *message)
 {
-    char   *myname = "qmgr_active_done";
+    const char *myname = "qmgr_active_done";
     struct stat st;
 
     if (msg_verbose)
@@ -331,7 +338,7 @@ static void qmgr_active_done_2_bounce_flush(int status, char *context)
 
 static void qmgr_active_done_2_generic(QMGR_MESSAGE *message)
 {
-    char   *myname = "qmgr_active_done_2_generic";
+    const char *myname = "qmgr_active_done_2_generic";
     const char *path;
     struct stat st;
     int     status;
@@ -403,7 +410,7 @@ static void qmgr_active_done_2_generic(QMGR_MESSAGE *message)
      * daemon waits for the qmgr to accept the "new mail" trigger.
      */
     if (message->flags) {
-	if (event_time() >= message->arrival_time +
+	if (event_time() >= message->create_time +
 	    (*message->sender ? var_max_queue_time : var_dsn_queue_time)) {
 	    msg_info("%s: from=<%s>, status=expired, returned to sender",
 		     message->queue_id, message->sender);
@@ -483,7 +490,7 @@ static void qmgr_active_done_3_defer_flush(int status, char *context)
 
 static void qmgr_active_done_3_generic(QMGR_MESSAGE *message)
 {
-    char   *myname = "qmgr_active_done_3_generic";
+    const char *myname = "qmgr_active_done_3_generic";
     int     delay;
 
     /*
@@ -498,8 +505,8 @@ static void qmgr_active_done_3_generic(QMGR_MESSAGE *message)
      * queue scans is finite.
      */
     if (message->flags) {
-	if (message->arrival_time > 0) {
-	    delay = event_time() - message->arrival_time;
+	if (message->create_time > 0) {
+	    delay = event_time() - message->create_time;
 	    if (delay > var_max_backoff_time)
 		delay = var_max_backoff_time;
 	    if (delay < var_min_backoff_time)
