@@ -128,18 +128,34 @@
 
 void    smtpd_peer_init(SMTPD_STATE *state)
 {
-    char   *myname = "smtpd_peer_init";
-    SOCKADDR_SIZE sa_len;
+    const char *myname = "smtpd_peer_init";
+    SOCKADDR_SIZE sa_length;
     struct sockaddr *sa;
     INET_PROTO_INFO *proto_info = inet_proto_info();
 
     sa = (struct sockaddr *) & (state->sockaddr);
-    sa_len = sizeof(state->sockaddr);
+    sa_length = sizeof(state->sockaddr);
 
     /*
      * Look up the peer address information.
+     * 
+     * XXX If we make local endpoint (getsockname) information available to
+     * Milter applications as {if_name} and {if_addr}, then we also must be
+     * able to provide this via the XCLIENT command for Milter testing.
+     * 
+     * XXX If we make local or remote port information available to policy
+     * servers or Milter applications, then we must also make this testable
+     * with the XCLIENT command, otherwise there will be confusion.
+     * 
+     * XXX If we make local or remote port information available via logging,
+     * then we must also support these attributes with the XFORWARD command.
+     * 
+     * XXX If support were to be added for Milter applications in down-stream
+     * MTAs, then consistency demands that we propagate a lot of Sendmail
+     * macro information via the XFORWARD command. Otherwise we could end up
+     * with a very confusing situation.
      */
-    if (getpeername(vstream_fileno(state->client), sa, &sa_len) >= 0) {
+    if (getpeername(vstream_fileno(state->client), sa, &sa_length) >= 0) {
 	errno = 0;
     }
 
@@ -151,6 +167,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
 	state->addr = mystrdup(CLIENT_ADDR_UNKNOWN);
 	state->rfc_addr = mystrdup(CLIENT_ADDR_UNKNOWN);
+	state->addr_family = AF_UNSPEC;
 	state->name_status = SMTPD_PEER_CODE_PERM;
 	state->reverse_name_status = SMTPD_PEER_CODE_PERM;
     }
@@ -179,7 +196,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	/*
 	 * Convert the client address to printable form.
 	 */
-	if ((aierr = sockaddr_to_hostaddr(sa, sa_len, &client_addr,
+	if ((aierr = sockaddr_to_hostaddr(sa, sa_length, &client_addr,
 					  (MAI_SERVPORT_STR *) 0, 0)) != 0)
 	    msg_fatal("%s: cannot convert client address to string: %s",
 		      myname, MAI_STRERROR(aierr));
@@ -203,14 +220,15 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 
 		state->addr = mystrdup(colonp + 1);
 		state->rfc_addr = mystrdup(colonp + 1);
+		state->addr_family = AF_INET;
 		aierr = hostaddr_to_sockaddr(state->addr, (char *) 0, 0, &res0);
 		if (aierr)
 		    msg_fatal("%s: cannot convert %s from string to binary: %s",
 			      myname, state->addr, MAI_STRERROR(aierr));
-		sa_len = res0->ai_addrlen;
-		if (sa_len > sizeof(state->sockaddr))
-		    sa_len = sizeof(state->sockaddr);
-		memcpy((char *) sa, res0->ai_addr, sa_len);
+		sa_length = res0->ai_addrlen;
+		if (sa_length > sizeof(state->sockaddr))
+		    sa_length = sizeof(state->sockaddr);
+		memcpy((char *) sa, res0->ai_addr, sa_length);
 		freeaddrinfo(res0);		/* 200412 */
 	    }
 
@@ -226,6 +244,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 		state->addr = mystrdup(client_addr.buf);
 		state->rfc_addr =
 		    concatenate(IPV6_COL, client_addr.buf, (char *) 0);
+		state->addr_family = sa->sa_family;
 	    }
 	}
 
@@ -237,6 +256,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	{
 	    state->addr = mystrdup(client_addr.buf);
 	    state->rfc_addr = mystrdup(client_addr.buf);
+	    state->addr_family = sa->sa_family;
 	}
 
 	/*
@@ -262,7 +282,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	    state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
 	    state->name_status = SMTPD_PEER_CODE_PERM;
 	    state->reverse_name_status = SMTPD_PEER_CODE_PERM;
-	} else if ((aierr = sockaddr_to_hostname(sa, sa_len, &client_name,
+	} else if ((aierr = sockaddr_to_hostname(sa, sa_length, &client_name,
 					 (MAI_SERVNAME_STR *) 0, 0)) != 0) {
 	    state->name = mystrdup(CLIENT_NAME_UNKNOWN);
 	    state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
@@ -290,13 +310,13 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 		msg_warn("%s: hostname %s verification failed: %s",
 			 state->addr, state->name, MAI_STRERROR(aierr));
 		REJECT_PEER_NAME(state, (TEMP_AI_ERROR(aierr) ?
-			      SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_PERM));
+			    SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_FORGED));
 	    } else {
 		for (res = res0; /* void */ ; res = res->ai_next) {
 		    if (res == 0) {
 			msg_warn("%s: address not listed for hostname %s",
 				 state->addr, state->name);
-			REJECT_PEER_NAME(state, SMTPD_PEER_CODE_PERM);
+			REJECT_PEER_NAME(state, SMTPD_PEER_CODE_FORGED);
 			break;
 		    }
 		    if (strchr((char *) proto_info->sa_family_list, res->ai_family) == 0) {
@@ -321,6 +341,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	state->reverse_name = mystrdup("localhost");
 	state->addr = mystrdup("127.0.0.1");	/* XXX bogus. */
 	state->rfc_addr = mystrdup("127.0.0.1");/* XXX bogus. */
+	state->addr_family = AF_UNSPEC;
 	state->name_status = SMTPD_PEER_CODE_OK;
 	state->reverse_name_status = SMTPD_PEER_CODE_OK;
     }
