@@ -58,8 +58,9 @@
 /*	int	vstream_fflush(stream)
 /*	VSTREAM	*stream;
 /*
-/*	int	vstream_fpurge(stream)
+/*	int	vstream_fpurge(stream, direction)
 /*	VSTREAM	*stream;
+/*	int     direction;
 /*
 /*	ssize_t	vstream_fread(stream, buf, len)
 /*	VSTREAM	*stream;
@@ -160,6 +161,7 @@
 /*
 /*	vstream_fclose() closes the named buffered stream. The result
 /*	is 0 in case of success, VSTREAM_EOF in case of problems.
+/*	vstream_fclose() reports the same errors as vstream_ferror().
 /*
 /*	vstream_fdclose() leaves the file(s) open but is otherwise
 /*	identical to vstream_fclose().
@@ -215,12 +217,15 @@
 /*	opened in read-write or write-only mode.
 /*	vstream_fflush() returns 0 in case of success, VSTREAM_EOF in
 /*	case of problems. It is an error to flush a read-only stream.
+/*	vstream_fflush() reports the same errors as vstream_ferror().
 /*
 /*	vstream_fpurge() discards the contents of the stream buffer.
-/*	In the case of a double-buffered stream, it discards the
-/*	content of both the read and write buffers.
-/*	vstream_fpurge() returns 0 in case of success, VSTREAM_EOF in
-/*	case of problems.
+/*	If direction is VSTREAM_PURGE_READ, it discards unread data,
+/*	else if direction is VSTREAM_PURGE_WRITE, it discards unwritten
+/*	data. In the case of a double-buffered stream, if direction is
+/*	VSTREAM_PURGE_BOTH, it discards the content of both the read
+/*	and write buffers. vstream_fpurge() returns 0 in case of success,
+/*	VSTREAM_EOF in case of problems.
 /*
 /*	vstream_fread() and vstream_fwrite() perform unformatted I/O
 /*	on the named stream. The result value is the number of bytes
@@ -255,6 +260,12 @@
 /*	The argument specifies the file descriptor to be used for writing.
 /*	This feature is limited to double-buffered streams, and makes the
 /*	stream non-seekable.
+/* .IP "VSTREAM_CTL_DUPFD (int)"
+/*	The argument specifies a minimum file descriptor value. If
+/*	the actual stream's file descriptors are below the minimum,
+/*	reallocate the descriptors to the first free value greater
+/*	than or equal to the minimum. The VSTREAM_CTL_DUPFD macro
+/*	is defined only on systems with fcntl() F_DUPFD support.
 /* .IP "VSTREAM_CTL_WAITPID_FN (int (*)(pid_t, WAIT_STATUS_T *, int))"
 /*	A pointer to function that behaves like waitpid(). This information
 /*	is used by the vstream_pclose() routine.
@@ -280,12 +291,18 @@
 /*
 /*	vstream_feof() returns non-zero when a previous operation on the
 /*	specified stream caused an end-of-file condition.
+/*	Although further read requests after EOF may complete
+/*	succesfully, vstream_feof() will keep returning non-zero
+/*	until vstream_clearerr() is called for that stream.
 /*
 /*	vstream_ferror() returns non-zero when a previous operation on the
 /*	specified stream caused a non-EOF error condition, including timeout.
+/*	After a non-EOF error on a stream, no I/O request will
+/*	complete until after vstream_clearerr() is called for that stream.
 /*
 /*	vstream_ftimeout() returns non-zero when a previous operation on the
-/*	specified stream caused a timeout error condition.
+/*	specified stream caused a timeout error condition. See
+/*	vstream_ferror() for error persistence details.
 /*
 /*	vstream_clearerr() resets the timeout, error and end-of-file indication
 /*	of the specified stream, and returns no useful result.
@@ -478,7 +495,7 @@ static void vstream_buf_init(VBUF *bp, int flags)
 static void vstream_buf_alloc(VBUF *bp, ssize_t len)
 {
     ssize_t used = bp->ptr - bp->data;
-    char   *myname = "vstream_buf_alloc";
+    const char *myname = "vstream_buf_alloc";
 
     if (len < bp->len)
 	msg_panic("%s: attempt to shrink buffer", myname);
@@ -512,7 +529,7 @@ static void vstream_buf_wipe(VBUF *bp)
 
 static int vstream_fflush_some(VSTREAM *stream, ssize_t to_flush)
 {
-    char   *myname = "vstream_fflush_some";
+    const char *myname = "vstream_fflush_some";
     VBUF   *bp = &stream->buf;
     ssize_t used;
     ssize_t left_over;
@@ -625,7 +642,7 @@ static int vstream_fflush_delayed(VSTREAM *stream)
 static int vstream_buf_get_ready(VBUF *bp)
 {
     VSTREAM *stream = VBUF_TO_APPL(bp, VSTREAM, buf);
-    char   *myname = "vstream_buf_get_ready";
+    const char *myname = "vstream_buf_get_ready";
     ssize_t n;
 
     /*
@@ -718,7 +735,7 @@ static int vstream_buf_get_ready(VBUF *bp)
 static int vstream_buf_put_ready(VBUF *bp)
 {
     VSTREAM *stream = VBUF_TO_APPL(bp, VSTREAM, buf);
-    char   *myname = "vstream_buf_put_ready";
+    const char *myname = "vstream_buf_put_ready";
 
     /*
      * Sanity checks. Detect a change of I/O direction or position. If so,
@@ -767,7 +784,7 @@ static int vstream_buf_space(VBUF *bp, ssize_t want)
     ssize_t used;
     ssize_t incr;
     ssize_t shortage;
-    char   *myname = "vstream_buf_space";
+    const char *myname = "vstream_buf_space";
 
     /*
      * Sanity checks. Reserving space implies writing. It is illegal to write
@@ -818,11 +835,16 @@ static int vstream_buf_space(VBUF *bp, ssize_t want)
 
 /* vstream_fpurge - discard unread or unwritten content */
 
-int vstream_fpurge(VSTREAM *stream)
+int     vstream_fpurge(VSTREAM *stream, int direction)
 {
     const char *myname = "vstream_fpurge";
     VBUF   *bp = &stream->buf;
 
+#define VSTREAM_MAYBE_PURGE_WRITE(d, b) if ((d) & VSTREAM_PURGE_WRITE) \
+	VSTREAM_BUF_AT_START((b))
+#define VSTREAM_MAYBE_PURGE_READ(d, b) if ((d) & VSTREAM_PURGE_READ) \
+	VSTREAM_BUF_AT_END((b))
+
     /*
      * To discard all unread contents, position the read buffer at its end,
      * so that we skip over any unread data, and so that the next read
@@ -834,20 +856,20 @@ int vstream_fpurge(VSTREAM *stream)
      */
     switch (bp->flags & (VSTREAM_FLAG_READ_DOUBLE | VSTREAM_FLAG_WRITE)) {
     case VSTREAM_FLAG_READ_DOUBLE:
-	VSTREAM_BUF_AT_START(&stream->write_buf);
+	VSTREAM_MAYBE_PURGE_WRITE(direction, &stream->write_buf);
 	/* FALLTHROUGH */
     case VSTREAM_FLAG_READ:
-	VSTREAM_BUF_AT_END(bp);
+	VSTREAM_MAYBE_PURGE_READ(direction, bp);
 	break;
     case VSTREAM_FLAG_DOUBLE:
-	VSTREAM_BUF_AT_START(&stream->write_buf);
-	VSTREAM_BUF_AT_END(&stream->read_buf);
+	VSTREAM_MAYBE_PURGE_WRITE(direction, &stream->write_buf);
+	VSTREAM_MAYBE_PURGE_READ(direction, &stream->read_buf);
 	break;
     case VSTREAM_FLAG_WRITE_DOUBLE:
-	VSTREAM_BUF_AT_END(&stream->read_buf);
+	VSTREAM_MAYBE_PURGE_READ(direction, &stream->read_buf);
 	/* FALLTHROUGH */
     case VSTREAM_FLAG_WRITE:
-	VSTREAM_BUF_AT_START(bp);
+	VSTREAM_MAYBE_PURGE_WRITE(direction, bp);
 	break;
     case VSTREAM_FLAG_READ_DOUBLE | VSTREAM_FLAG_WRITE:
     case VSTREAM_FLAG_READ | VSTREAM_FLAG_WRITE:
@@ -867,7 +889,7 @@ int vstream_fpurge(VSTREAM *stream)
 
 off_t   vstream_fseek(VSTREAM *stream, off_t offset, int whence)
 {
-    char   *myname = "vstream_fseek";
+    const char *myname = "vstream_fseek";
     VBUF   *bp = &stream->buf;
 
     /*
@@ -1049,8 +1071,9 @@ int     vstream_fclose(VSTREAM *stream)
      */
     if (stream->pid != 0)
 	msg_panic("vstream_fclose: stream has process");
-    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0)
+    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0 && stream->fd >= 0)
 	vstream_fflush(stream);
+    /* Do not remove: vstream_fdclose() depends on this error test. */
     err = vstream_ferror(stream);
     if (stream->buf.flags & VSTREAM_FLAG_DOUBLE) {
 	if (stream->read_fd >= 0)
@@ -1079,8 +1102,22 @@ int     vstream_fclose(VSTREAM *stream)
 
 int     vstream_fdclose(VSTREAM *stream)
 {
+
+    /*
+     * Flush unwritten output, just like vstream_fclose(). Errors are
+     * reported by vstream_fclose().
+     */
+    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0)
+	(void) vstream_fflush(stream);
+
+    /*
+     * NOTE: Negative file descriptors are not part of the external
+     * interface. They are for internal use only, in order to support
+     * vstream_fdclose() without a lot of code duplication. Applications that
+     * rely on negative VSTREAM file descriptors will break without warning.
+     */
     if (stream->buf.flags & VSTREAM_FLAG_DOUBLE) {
-	stream->read_fd = stream->write_fd = -1;
+	stream->fd = stream->read_fd = stream->write_fd = -1;
     } else {
 	stream->fd = -1;
     }
@@ -1128,8 +1165,10 @@ int     vstream_fputs(const char *str, VSTREAM *stream)
 
 void    vstream_control(VSTREAM *stream, int name,...)
 {
-    char   *myname = "vstream_control";
+    const char *myname = "vstream_control";
     va_list ap;
+    int     floor;
+    int     old_fd;
 
     for (va_start(ap, name); name != VSTREAM_CTL_END; name = va_arg(ap, int)) {
 	switch (name) {
@@ -1183,6 +1222,32 @@ void    vstream_control(VSTREAM *stream, int name,...)
 	    if (stream->jbuf == 0)
 		stream->jbuf = (jmp_buf *) mymalloc(sizeof(jmp_buf));
 	    break;
+
+#ifdef VSTREAM_CTL_DUPFD
+
+#define VSTREAM_TRY_DUPFD(backup, fd, floor) do { \
+	if (((backup) = (fd)) < floor) { \
+	    if (((fd) = fcntl((backup), F_DUPFD, (floor))) < 0) \
+		msg_fatal("fcntl F_DUPFD %d: %m", (floor)); \
+	    (void) close(backup); \
+	} \
+    } while (0)
+
+	case VSTREAM_CTL_DUPFD:
+	    floor = va_arg(ap, int);
+	    if (stream->buf.flags & VSTREAM_FLAG_DOUBLE) {
+		VSTREAM_TRY_DUPFD(old_fd, stream->read_fd, floor);
+		if (stream->write_fd == old_fd)
+		    stream->write_fd = stream->read_fd;
+		else
+		    VSTREAM_TRY_DUPFD(old_fd, stream->write_fd, floor);
+		stream->fd = (stream->buf.flags & VSTREAM_FLAG_READ) ?
+		    stream->read_fd : stream->write_fd;
+	    } else {
+		VSTREAM_TRY_DUPFD(old_fd, stream->fd, floor);
+	    }
+	    break;
+#endif
 	default:
 	    msg_panic("%s: bad name %d", myname, name);
 	}
